@@ -18,7 +18,7 @@ from narwhals.typing import IntoDataFrame
 
 import marimo._output.data.data as mo_data
 from marimo import _loggers
-from marimo._data.models import ColumnStats
+from marimo._data.models import BinValue, ColumnStats, ValueCount
 from marimo._data.preview_column import get_column_preview_dataset
 from marimo._dependencies.dependencies import DependencyManager
 from marimo._messaging.ops import ColumnPreview
@@ -84,15 +84,26 @@ class DownloadAsArgs:
     format: Literal["csv", "json", "parquet"]
 
 
+@dataclass
+class ColumnSummariesArgs:
+    """If enabled, we will precompute chart values."""
+
+    precompute: bool
+
+
 @dataclass
 class ColumnSummaries:
     data: Union[JSONType, str]
     stats: dict[ColumnName, ColumnStats]
+    bin_values: dict[ColumnName, list[BinValue]]
+    value_counts: dict[ColumnName, list[ValueCount]]
     # Disabled because of too many columns/rows
     # This will show a banner in the frontend
     is_disabled: Optional[bool] = None
 
 
+ShowColumnSummaries = Union[bool, Literal["stats", "chart"]]
+
 DEFAULT_MAX_COLUMNS = 50
 
 MaxColumnsNotProvided = Literal["inherit"]
@@ -306,6 +317,8 @@ class table(
         show_column_summaries (Union[bool, Literal["stats", "chart"]], optional): Whether to show column summaries.
             Defaults to True when the table has less than 40 columns and at least 10 rows, False otherwise.
             If "stats", only show stats. If "chart", only show charts.
+        show_data_types (bool, optional): Whether to show data types of columns in the table header.
+            Defaults to True.
         show_download (bool, optional): Whether to show the download button.
             Defaults to True for dataframes, False otherwise.
         format_mapping (Dict[str, Union[str, Callable[..., Any]]], optional): A mapping from
@@ -397,9 +410,8 @@ class table(
             Union[list[int], list[tuple[str, str]]]
         ] = None,
         page_size: Optional[int] = None,
-        show_column_summaries: Optional[
-            Union[bool, Literal["stats", "chart"]]
-        ] = None,
+        show_column_summaries: Optional[ShowColumnSummaries] = None,
+        show_data_types: bool = True,
         format_mapping: Optional[
             dict[str, Union[str, Callable[..., Any]]]
         ] = None,
@@ -487,7 +499,9 @@ class table(
                     >= TableManager.DEFAULT_SUMMARY_CHARTS_MINIMUM_ROWS
                 )
             )
-        self._show_column_summaries = show_column_summaries
+        self._show_column_summaries: ShowColumnSummaries = (
+            show_column_summaries
+        )
 
         if _internal_column_charts_row_limit is not None:
             self._column_charts_row_limit = _internal_column_charts_row_limit
@@ -641,6 +655,7 @@ class table(
                 "show-download": show_download
                 and self._manager.supports_download(),
                 "show-column-summaries": show_column_summaries,
+                "show-data-types": show_data_types,
                 "show-page-size-selector": show_page_size_selector,
                 "show-column-explorer": show_column_explorer,
                 "show-chart-builder": show_chart_builder,
@@ -663,7 +678,7 @@ class table(
                 ),
                 Function(
                     name="get_column_summaries",
-                    arg_cls=EmptyArgs,
+                    arg_cls=ColumnSummariesArgs,
                     function=self._get_column_summaries,
                 ),
                 Function(
@@ -793,7 +808,9 @@ class table(
                 "Download is not supported for this table format."
             )
 
-    def _get_column_summaries(self, args: EmptyArgs) -> ColumnSummaries:
+    def _get_column_summaries(
+        self, args: ColumnSummariesArgs
+    ) -> ColumnSummaries:
         """Get statistical summaries for each column in the table.
 
         Calculates summaries like null counts, min/max values, unique counts, etc.
@@ -801,18 +818,22 @@ class table(
         is below the column summary row limit.
 
         Args:
-            args (EmptyArgs): Empty arguments object (unused).
+            args (ColumnSummariesArgs): Arguments specifying whether to precompute
+                the column summaries and bin values.
 
         Returns:
             ColumnSummaries: Object containing column summaries and chart data.
                 If summaries are disabled or row limit is exceeded, returns empty
                 summaries with is_disabled flag set appropriately.
         """
-        del args
-        if not self._show_column_summaries:
+        show_column_summaries = self._show_column_summaries
+
+        if not show_column_summaries:
             return ColumnSummaries(
                 data=None,
                 stats={},
+                bin_values={},
+                value_counts={},
                 # This is not 'disabled' because of too many rows
                 # so we don't want to display the banner
                 is_disabled=False,
@@ -826,13 +847,34 @@ class table(
             return ColumnSummaries(
                 data=None,
                 stats={},
+                bin_values={},
+                value_counts={},
                 is_disabled=True,
             )
 
+        # If we are above the limit to show charts,
+        # or if we are in stats-only mode,
+        # we don't return the chart data
+        should_get_chart_data = (
+            self._show_column_summaries != "stats"
+            and total_rows <= self._column_charts_row_limit
+        )
+
         # Get column stats if not chart-only mode
+        should_get_stats = show_column_summaries != "chart"
         stats: dict[ColumnName, ColumnStats] = {}
-        if self._show_column_summaries != "chart":
-            for column in self._manager.get_column_names():
+
+        chart_data = None
+        bin_values: dict[ColumnName, list[BinValue]] = {}
+        value_counts: dict[ColumnName, list[ValueCount]] = {}
+        data = self._searched_manager
+
+        DEFAULT_BIN_SIZE = 9
+        DEFAULT_VALUE_COUNTS_SIZE = 15
+
+        for column in self._manager.get_column_names():
+            statistic = None
+            if should_get_stats:
                 try:
                     statistic = self._searched_manager.get_stats(column)
                     stats[column] = statistic
@@ -841,22 +883,120 @@ class table(
                     # BaseExceptions, which shouldn't crash the kernel
                     LOGGER.warning("Failed to get stats for column %s", column)
 
-        # If we are above the limit to show charts,
-        # or if we are in stats-only mode,
-        # we don't return the chart data
-        chart_data = None
-        if (
-            self._show_column_summaries != "stats"
-            and total_rows <= self._column_charts_row_limit
-        ):
-            chart_data, _ = self._to_chart_data_url(self._searched_manager)
+            if should_get_chart_data and args.precompute:
+                if not should_get_stats:
+                    LOGGER.warning(
+                        "Unable to compute stats for column, may not be computed correctly"
+                    )
+
+                # For boolean columns, we can drop the column since we use stats
+                column_type = self._manager.get_field_type(column)
+                if column_type[0] == "boolean":
+                    data = data.drop_columns([column])
+
+                # Bin values are only supported for numeric and temporal columns
+                if column_type[0] not in [
+                    "integer",
+                    "number",
+                    "date",
+                    "datetime",
+                    "time",
+                    "string",
+                ]:
+                    continue
+
+                # For perf, we only compute value counts for categorical columns
+                external_type = column_type[1].lower()
+                if column_type[0] == "string" and (
+                    "cat" in external_type or "enum" in external_type
+                ):
+                    try:
+                        val_counts = self._get_value_counts(
+                            column, DEFAULT_VALUE_COUNTS_SIZE, total_rows
+                        )
+                        if len(val_counts) > 0:
+                            value_counts[column] = val_counts
+                            data = data.drop_columns([column])
+                        continue
+                    except BaseException as e:
+                        LOGGER.warning(
+                            "Failed to get value counts for column %s: %s",
+                            column,
+                            e,
+                        )
+
+                try:
+                    if statistic and statistic.nulls == total_rows:
+                        bins = []
+                    else:
+                        bins = data.get_bin_values(column, DEFAULT_BIN_SIZE)
+                    bin_values[column] = bins
+                    data = data.drop_columns([column])
+                    continue
+                except BaseException as e:
+                    LOGGER.warning(
+                        "Failed to get bin values for column %s: %s", column, e
+                    )
+
+        if should_get_chart_data:
+            chart_data, _ = self._to_chart_data_url(data)
 
         return ColumnSummaries(
             data=chart_data,
             stats=stats,
+            bin_values=bin_values,
+            value_counts=value_counts,
             is_disabled=False,
         )
 
+    def _get_value_counts(
+        self, column: ColumnName, size: int, total_rows: int
+    ) -> list[ValueCount]:
+        """Get value counts for a column. The last item will be 'others' with the count of remaining
+        unique values. If there are only unique values, we return 'unique values' instead.
+
+        Args:
+            column (ColumnName): The column to get value counts for.
+            size (int): The number of value counts to return.
+            total_rows (int): The total number of rows in the table.
+
+        Returns:
+            list[ValueCount]: The value counts.
+        """
+        if size <= 0 or total_rows <= 0:
+            LOGGER.warning("Total rows and size is not valid")
+            return []
+
+        top_k_rows = self._manager.calculate_top_k_rows(column, size)
+        if len(top_k_rows) == 0:
+            return []
+
+        all_unique = top_k_rows[0][1] == 1
+        if all_unique:
+            return [ValueCount(value="unique values", count=total_rows)]
+
+        value_counts = []
+
+        if len(top_k_rows) == size:
+            # reserve 1 for others
+            top_k_rows = top_k_rows[:-1]
+
+        sum_count = 0
+        for value, count in top_k_rows:
+            sum_count += count
+            value = str(value) if value is not None else "null"
+            value_counts.append(ValueCount(value=value, count=count))
+
+        remaining = total_rows - sum_count
+        if remaining > 0:
+            unique_count = ValueCount(value="others", count=remaining)
+            if len(value_counts) == size:
+                value_counts[-1] = unique_count
+            else:
+                value_counts.append(unique_count)
+
+        return value_counts
+
     @classmethod
     def _to_chart_data_url(
         cls, table_manager: TableManager[Any]
@@ -870,16 +1010,35 @@ class table(
         We return a URL instead of the data directly
         so the browser can cache requests
         """
-        try:
-            data_url = mo_data.arrow(table_manager.to_arrow_ipc()).url
-            return data_url, "arrow"
-        except NotImplementedError:
+        if DependencyManager.pyarrow.has():
             try:
-                data_url = mo_data.csv(table_manager.to_csv({})).url
-                return data_url, "csv"
-            except ValueError:
-                data_url = mo_data.json(table_manager.to_json({})).url
-                return data_url, "json"
+                data_url = mo_data.arrow(table_manager.to_arrow_ipc()).url
+                return data_url, "arrow"
+            except NotImplementedError:
+                LOGGER.debug(
+                    "Arrow export not implemented, falling back to CSV."
+                )
+            except Exception as e:
+                LOGGER.error("Unexpected error exporting Arrow: %s", e)
+
+        # Try CSV
+        try:
+            data_url = mo_data.csv(table_manager.to_csv({})).url
+            return data_url, "csv"
+        except (ValueError, NotImplementedError):
+            LOGGER.debug("CSV export failed, falling back to JSON.")
+        except Exception as e:
+            LOGGER.error("Unexpected error exporting CSV: %s", e)
+
+        # Fallback to JSON
+        try:
+            data_url = mo_data.json(table_manager.to_json({})).url
+            return data_url, "json"
+        except Exception as e:
+            LOGGER.error(
+                "Failed to export table data as Arrow, CSV, or JSON: %s", e
+            )
+            raise
 
     def _get_data_url(self, args: EmptyArgs) -> GetDataUrlResponse:
         """Get the data URL for the entire table. Used for charting."""
@@ -917,17 +1076,26 @@ class table(
         result = self._manager
 
         if filters:
-            data = unwrap_narwhals_dataframe(result.data)
-            handler = get_handler_for_dataframe(data)
-            data = handler.handle_filter_rows(
-                data,
-                FilterRowsTransform(
-                    type=TransformType.FILTER_ROWS,
-                    where=filters,
-                    operation="keep_rows",
-                ),
-            )
-            result = get_table_manager(data)
+            # Filter out conditions for columns that don't exist
+            existing_columns = set(result.get_column_names())
+            valid_filters = [
+                condition
+                for condition in filters
+                if condition.column_id in existing_columns
+            ]
+
+            if valid_filters:
+                data = unwrap_narwhals_dataframe(result.data)
+                handler = get_handler_for_dataframe(data)
+                data = handler.handle_filter_rows(
+                    data,
+                    FilterRowsTransform(
+                        type=TransformType.FILTER_ROWS,
+                        where=valid_filters,
+                        operation="keep_rows",
+                    ),
+                )
+                result = get_table_manager(data)
 
         if query:
             result = result.search(query)
