@@ -7,7 +7,7 @@ from collections import defaultdict
 from collections.abc import Sequence
 from typing import Any, Optional, Union, cast
 
-from marimo._data.models import ColumnStats, ExternalDataType
+from marimo._data.models import BinValue, ColumnStats, ExternalDataType
 from marimo._dependencies.dependencies import DependencyManager
 from marimo._output.mime import MIME
 from marimo._plugins.core.json_encoder import WebComponentEncoder
@@ -256,7 +256,7 @@ class DefaultTableManager(TableManager[JsonTableData]):
             ]
         )
 
-    def get_row_headers(self) -> list[str]:
+    def get_row_headers(self) -> FieldTypes:
         return []
 
     @functools.lru_cache(maxsize=5)  # noqa: B019
@@ -328,6 +328,10 @@ class DefaultTableManager(TableManager[JsonTableData]):
         del column
         return ColumnStats()
 
+    def get_bin_values(self, column: str, num_bins: int) -> list[BinValue]:
+        del column, num_bins
+        return []
+
     def get_num_rows(self, force: bool = True) -> int:
         del force
         if isinstance(self.data, dict):
