@@ -152,6 +152,11 @@ class DirectedGraph:
             self.children[cell_id] = children
             self.siblings[cell_id] = siblings
             self.parents[cell_id] = parents
+
+            # First, we process the variables that this cell defines. Any cell
+            # that refers to a defined variable becomes a child of this cell;
+            # any cell that defines a variable defined by this cell becomes
+            # a sibling.
             for name, variable_data in cell.variable_data.items():
                 self.definitions.setdefault(name, set()).add(cell_id)
                 for sibling in self.definitions[name]:
@@ -178,15 +183,21 @@ class DirectedGraph:
                 for child in referring_cells:
                     self.parents[child].add(cell_id)
 
+            # Next, we process the cells references. The cell becomes a child
+            # of cells that define its referenced variables. We also have
+            # special logic for handling references that are deleted by this cell,
+            # since cells that delete variables that were defined elsewhere
+            # are made children of cells that reference that variable.
             for name in cell.refs:
+                # First, for each referenced variable, we add cells that define
+                # that variable as parents
                 other_ids_defining_name: set[CellId_t] = (
                     self.definitions[name]
                     if name in self.definitions
                     else set()
                 ) - set((cell_id,))
-                # if other is empty, this means that the user is going to
-                # get a NameError once the cell is run, unless the symbol
-                # is say a builtin
+                # If other_ids_defining_name is empty, the user will get a
+                # NameError at runtime (unless the symbol is a builtin).
                 for other_id in other_ids_defining_name:
                     language = (
                         self.cells[other_id].variable_data[name][-1].language
@@ -202,6 +213,53 @@ class DirectedGraph:
                     if path:
                         self.cycles.add(tuple([(other_id, cell_id)] + path))
                     self.children[other_id].add(cell_id)
+
+                # Next, any cell that deletes this referenced variable is made
+                # a child of this cell. In particular, if a cell deletes a
+                # variable, it becomes a child of all other cells that
+                # reference that variable. This means that if two cells delete
+                # the same variable, they form a cycle.
+                #
+                # For example, two cells
+                #
+                #   cell u: x
+                #   cell v: del x
+                #
+                # v becomes a child of u.
+                #
+                # Another example:
+                #
+                #   cell u: del x
+                #   cell v: del x
+                #
+                # u and v form a cycle.
+                other_ids_deleting_name: set[CellId_t] = set(
+                    cid
+                    for cid in self.get_referring_cells(
+                        name, language="python"
+                    )
+                    if name in self.cells[cid].deleted_refs
+                ) - set((cell_id,))
+                for v in other_ids_deleting_name:
+                    path = self.get_path(v, cell_id)
+                    if path:
+                        self.cycles.add(tuple([(cell_id, v)] + path))
+                    self.parents[v].add(cell_id)
+                children.update(other_ids_deleting_name)
+
+            # Finally, if this cell deletes a variable, we make it a child of
+            # all other cells that reference this variable.
+            for name in cell.deleted_refs:
+                referring_cells = self.get_referring_cells(
+                    name, language="python"
+                ) - set((cell_id,))
+                for other_id in referring_cells:
+                    parents.add(other_id)
+                    path = self.get_path(cell_id, other_id)
+                    if path:
+                        self.cycles.add(tuple([(other_id, cell_id)] + path))
+                    self.children[other_id].add(cell_id)
+
         LOGGER.debug("Registered cell %s and released graph lock", cell_id)
         if self.is_any_ancestor_stale(cell_id):
             self.set_stale(set([cell_id]))
