@@ -22,6 +22,7 @@ from typing import (
     overload,
 )
 
+from marimo._ast.cell_id import is_external_cell_id
 from marimo._ast.transformers import (
     ARG_PREFIX,
     CacheExtractWithBlock,
@@ -29,7 +30,7 @@ from marimo._ast.transformers import (
 )
 from marimo._ast.variables import is_mangled_local, unmangle_local
 from marimo._messaging.tracebacks import write_traceback
-from marimo._runtime.context import get_context
+from marimo._runtime.context import get_context, safe_get_context
 from marimo._runtime.side_effect import SideEffect
 from marimo._runtime.state import State
 from marimo._save.cache import Cache, CacheException
@@ -48,6 +49,7 @@ from marimo._save.loaders import (
     MemoryLoader,
 )
 from marimo._save.stores.file import FileStore
+from marimo._save.toplevel import get_cell_id_from_scope, graph_from_scope
 from marimo._types.ids import CellId_t
 from marimo._utils.with_skip import SkipContext
 
@@ -63,6 +65,7 @@ UNEXPECTED_FAILURE_BOILERPLATE = (
 if TYPE_CHECKING:
     from types import FrameType, TracebackType
 
+    from marimo._runtime.dataflow import DirectedGraph
     from marimo._save.stores import Store
 
 
@@ -83,10 +86,11 @@ class _cache_call:
         "_bound",
         "_last_hash",
         "_frame_offset",
+        "_external",
         "__wrapped__",
     )
 
-    base_block: BlockHasher
+    base_block: Optional[BlockHasher]
     scope: dict[str, Any]
     scoped_refs: set[str]
     pin_modules: bool
@@ -99,6 +103,7 @@ class _cache_call:
     _bound: Optional[dict[str, Any]]
     _last_hash: Optional[str]
     _frame_offset: int
+    _external: bool
     # Consistent with functools.cache
     __wrapped__: Optional[Callable[..., Any]]
 
@@ -113,6 +118,7 @@ class _cache_call:
         # with respect to definition of _fn
         frame_offset: int = 0,
     ) -> None:
+        self.base_block = None
         self.pin_modules = pin_modules
         self.hash_type = hash_type
         self._frame_offset = frame_offset
@@ -122,6 +128,7 @@ class _cache_call:
         self._var_kwarg = None
         self._loader = None
         self._bound = {}
+        self._external = False
         if _fn is None:
             self.__wrapped__ = None
         else:
@@ -134,13 +141,25 @@ class _cache_call:
         return self.loader.hits
 
     def _set_context(self, fn: Callable[..., Any]) -> None:
-        assert callable(fn), "the provided function must be callable"
-        ctx = get_context()
-        assert ctx.execution_context is not None, (
-            "Could not resolve context for cache. "
-            "Either @cache is not called from a top level cell or "
-            f"{UNEXPECTED_FAILURE_BOILERPLATE}"
-        )
+        ctx = safe_get_context()
+
+        # If we are loaded from a module, then we have no context.
+        # Default to this case for typing.
+        self._external = True
+        cell_id = CellId_t("")
+        graph: Optional[DirectedGraph] = None
+        glbls = {}
+        if ctx and ctx.execution_context is not None:
+            maybe_cell_id = (
+                ctx.cell_id or ctx.execution_context.cell_id or CellId_t("")
+            )
+            # If the cell ID is "external", that means it's not from the main
+            # graph but rather from an embedded graph.
+            self._external = is_external_cell_id(maybe_cell_id)
+            if not self._external:
+                graph = ctx.graph
+                glbls = ctx.globals
+                cell_id = maybe_cell_id
 
         self.__wrapped__ = fn
         sig = inspect.signature(fn)
@@ -180,32 +199,51 @@ class _cache_call:
         # Note, that deeply nested frames may cause issues, however
         # checking a single frame- should be good enough.
         f_locals = inspect.stack()[2 + self._frame_offset][0].f_locals
-        self.scope = {**ctx.globals, **f_locals}
+        if glbls:
+            self.scope = {**glbls, **f_locals}
+        else:
+            # Direct assignment, because we need the reference for later
+            # lookups.
+            self.scope = f_locals
 
         # Scoped refs are references particular to this block, that may not be
         # defined out of the context of the block, or the cell.
         # For instance, the args of the invoked function are restricted to the
         # block.
-        cell_id = ctx.cell_id or ctx.execution_context.cell_id or CellId_t("")
         self.scoped_refs = set([f"{ARG_PREFIX}{k}" for k in self._args])
         # As are the "locals" not in globals
-        self.scoped_refs |= set(f_locals.keys()) - set(ctx.globals.keys())
+        self.scoped_refs |= set(f_locals.keys()) - set(glbls.keys())
         # Defined in the cell, and currently available in scope
-        self.scoped_refs |= ctx.graph.cells[cell_id].defs & set(
-            ctx.globals.keys()
-        )
-        # The defined private variables of this cell, normalized
-        self.scoped_refs |= set(
-            unmangle_local(x).name
-            for x in ctx.globals.keys()
-            if is_mangled_local(x, cell_id)
-        )
+        if graph is not None:
+            self.scoped_refs |= graph.cells[cell_id].defs & set(glbls.keys())
+            # The defined private variables of this cell, normalized
+            self.scoped_refs |= set(
+                unmangle_local(x).name
+                for x in glbls.keys()
+                if is_mangled_local(x, cell_id)
+            )
 
-        graph = ctx.graph
-        cell_id = ctx.cell_id or ctx.execution_context.cell_id
-        module = strip_function(self.__wrapped__)
+        # Load global cache from state
+        name = self.__name__
+        # Note, that if the function name shadows a global variable, the
+        # lifetime of the cache will be tied to the global variable.
+        # We can invalidate that by making an invalid namespace.
+        if glbls != f_locals:
+            name = f"{name}*"
 
-        self.base_block = BlockHasher(
+        self._loader = self._loader_partial.create_or_reconfigure(name)
+
+        if graph is not None:
+            self.base_block = self._build_base_block(
+                self.__wrapped__, graph, cell_id
+            )
+
+    def _build_base_block(
+        self, fn: Callable[..., Any], graph: DirectedGraph, cell_id: CellId_t
+    ) -> BlockHasher:
+        module = strip_function(fn)
+
+        return BlockHasher(
             module=module,
             graph=graph,
             cell_id=cell_id,
@@ -214,27 +252,28 @@ class _cache_call:
             hash_type=self.hash_type,
             scoped_refs=self.scoped_refs,
             apply_content_hash=False,
+            external=self._external,
         )
 
-        # Load global cache from state
-        name = self.__wrapped__.__name__
-        # Note, that if the function name shadows a global variable, the
-        # lifetime of the cache will be tied to the global variable.
-        # We can invalidate that by making an invalid namespace.
-        if ctx.globals != f_locals:
-            name = f"{name}*"
-
-        self._loader = self._loader_partial.create_or_reconfigure(name)
-
     @property
     def loader(self) -> Loader:
         assert self._loader is not None, UNEXPECTED_FAILURE_BOILERPLATE
         return self._loader()
 
+    @property
+    def __name__(self) -> str:
+        """Return the name of the wrapped function."""
+        # NB. __name__ is expected on introspection in compiler.
+        if self.__wrapped__ is None:
+            return "<cache>"
+        return self.__wrapped__.__name__
+
     def __get__(
         self, instance: Any, _owner: Optional[type] = None
     ) -> _cache_call:
-        """__get__ is invoked on instance access;
+        """Enable @cache as a method decorator.
+
+        __get__ is invoked on instance access;
             e.g. `obj.fn` (__get__ called on `fn`)
         `instance` is the specific object, while owner is `type(instance)`.
 
@@ -286,6 +325,16 @@ class _cache_call:
             self._set_context(args[0])
             return self
 
+        if self.base_block is None:
+            assert self._external, UNEXPECTED_FAILURE_BOILERPLATE
+            # We only build the graph on invocation because toplevel functions
+            # can be defined out of order.
+            graph = graph_from_scope(self.scope)
+            cell_id = get_cell_id_from_scope(self.__wrapped__, self.scope)
+            self.base_block = self._build_base_block(
+                self.__wrapped__, graph, cell_id
+            )
+
         # Rewrite scoped args to prevent shadowed variables
         arg_dict = {f"{ARG_PREFIX}{k}": v for (k, v) in zip(self._args, args)}
         kwargs_copy = {f"{ARG_PREFIX}{k}": v for (k, v) in kwargs.items()}
@@ -297,10 +346,24 @@ class _cache_call:
             arg_dict[f"{ARG_PREFIX}{self._var_kwarg}"] = kwargs.copy()
 
         # Capture the call case
-        ctx = get_context()
+        ctx = safe_get_context()
+        glbls: dict[str, Any] = {}
+        if ctx is not None:
+            glbls = ctx.globals
+        # Typically, scope is overridden by globals (scope is just a snapshot of
+        # the current frame, which may have changed)- however in an external
+        # context, scope is the only source of glbls (the definition should be
+        # unaware of working memory).
         scope = {
             **self.scope,
-            **ctx.globals,
+        }
+        if not self._external:
+            scope = {
+                **scope,
+                **glbls,
+            }
+        scope = {
+            **scope,
             **arg_dict,
             **kwargs_copy,
             **(self._bound or {}),
@@ -333,7 +396,7 @@ class _cache_call:
             raise e
         finally:
             # NB. Exceptions raise their own side effects.
-            if not failed:
+            if ctx and not failed:
                 ctx.cell_lifecycle_registry.add(SideEffect(attempt.hash))
         return response
 
@@ -858,20 +921,6 @@ def persistent_cache(  # type: ignore[misc]
     **Warning.** Since context abuses sys frame trace, this may conflict with
     debugging tools or libraries that also use `sys.settrace`.
 
-    Args:
-        name: the name of the cache, used to set saving path- to manually
-            invalidate the cache, change the name.
-        save_path: the folder in which to save the cache, defaults to
-            `__marimo__/cache` in the directory of the notebook file
-        method: the serialization method to use, current options are "json",
-            and "pickle" (default).
-        pin_modules: if True, the cache will be invalidated if module versions
-            differ between runs, defaults to False.
-        store: optional store.
-        **kwargs: keyword arguments passed to `cache()`
-        *args: positional arguments passed to `cache()`
-
-
     ## Decorator for persistently caching the return value of a function.
 
     `persistent_cache` can also be used as a drop in function-level memoization
@@ -897,13 +946,18 @@ def persistent_cache(  # type: ignore[misc]
     ```
 
     Args:
-        fn: the wrapped function if no settings are passed.
+        name: the name of the cache, used to set saving path- to manually
+            invalidate the cache, change the name.
         save_path: the folder in which to save the cache, defaults to
             `__marimo__/cache` in the directory of the notebook file
         method: the serialization method to use, current options are "json",
             and "pickle" (default).
+        store: optional store.
+        fn: the wrapped function if no settings are passed.
+        *args: positional arguments passed to `cache()`
         pin_modules: if True, the cache will be invalidated if module versions
             differ between runs, defaults to False.
+        **kwargs: keyword arguments passed to `cache()`
     """
 
     arg = name
